package com.malk.laidi.controller;

import com.alibaba.fastjson.JSON;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_Contacts;
import com.malk.service.dingtalk.DDClient_Storage;
import com.malk.service.dingtalk.DDClient_Workflow;
import com.malk.service.h3yun.CYClient;
import com.malk.utils.UtilMap;
import com.malk.utils.UtilServlet;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 错误抛出与拦截详见 CatchException
 */
@Slf4j
@RestController
@RequestMapping()
public class LDController {

    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    @Autowired
    private CYClient cyClient;

    /// 匹配氚云用户ID [ ppExt 临时方案为, H_User系统表, 字段DingTalkAccount「返回格式为 userId.corpId」, 通过氚云定时任务储存与钉钉对照关系 ]
    String _getCYUserIdByDingTalkUserId(String dUserId) {
        Map extInfo = UtilMap.map("matcherJson", JSON.toJSONString(UtilMap.map("Type, Name, Operator, Value", "Item", "F0000003", 2, dUserId)));
        List<Map> userList = cyClient.queryData(ddClient.getAccessToken(), "D1489118960387db0274aa18b9168cde213c3d3", extInfo);
        return String.valueOf(userList.get(0).get("F0000002"));
    }

    /**
     * 同步OA出差单到氚云
     */
    @PostMapping("sync-cc")
    McR syncCC(@RequestBody Map data) {

        String processInstanceId = UtilMap.getString(data, "processInstanceId");
//        String processInstanceId = "KGkpcf8qRUe26Y4Vh7ICxQ08031691566909"; //
//        String processInstanceId = "ckDghK5NSHuLYBVammK4Cg08031694687962";

        Map processData = ddClient_workflow.getProcessInstanceId(ddClient.getAccessToken(), processInstanceId);
        List<Map> formComponentValues = (List<Map>) processData.get("formComponentValues");

        // OA组件name, 匹配氚云组件ID
        String schemaCode = "D148911abbb84666b4b481783235d0fcfc39d8d";
        Map<String, String> compsId_main = UtilMap.map("出差事由, 出差天数, 出差备注, 同行人", "F0000002, F0000010, F0000011, F0000015");
        Map<String, String> compsId_itinerary = UtilMap.map("交通工具, 单程往返, 出发城市, 目的城市, 开始时间, 结束时间, 时长", "F0000003, F0000004, F0000005, F0000006, F0000007, F0000008, F0000009");
        compsId_main.put("行程明细", "D148911F875802263af84ddaa01b9e6bd679eb74"); // 子表组件

        String dUserId = String.valueOf(processData.get("originatorUserId"));
        String cUserId = _getCYUserIdByDingTalkUserId(dUserId);
//        String cUserId = "fe68dfe1-27f7-4533-8a55-081590e4462a"; // 吴作江
        Map formData = UtilMap.map("F0000013", cUserId);

        for (String name : compsId_main.keySet()) {
            String compId = compsId_main.get(name);
            // 判定是否子表 [氚云]
            if (compId.startsWith("D")) {
                List<Map> details = new ArrayList<>();
                // ppExt: 出差申请单为明细组件, 存在多条情况
                String schedule = (String) formComponentValues.stream().filter(item -> "itinerary".equals(item.get("bizAlias"))).findAny().get().get("value");
                List<Map> itineraryList = ((List<Map>) JSON.parse(schedule));
                // 循环明细数据
                for (Map itinerary : itineraryList) {
                    List<Map> rowValue = (List<Map>) itinerary.get("rowValue");
                    Map rowData = new HashedMap();
                    // 循环子表组件
                    for (String subName : compsId_itinerary.keySet()) {
                        rowData.put(compsId_itinerary.get(subName), rowValue.stream().filter(item -> subName.equals(item.get("label"))).findAny().get().get("value"));
                    }
                    details.add(rowData);
                }
                formData.put(compId, details);
                continue;
            }
            Map formComp = formComponentValues.stream().filter(item -> name.equals(item.get("name"))).findAny().get();
            Object value = formComp.get("value");
            // 同行人, 数据处理
            if ("InnerContactField".equals(formComp.get("componentType")) && formComp.containsKey("value")) {
                List<Map> empInfos = (List<Map>) JSON.parse(String.valueOf(formComp.get("extValue")));
                List<String> emplsId = new ArrayList<>();
                for (Map empInfo : empInfos) {
                    emplsId.add(_getCYUserIdByDingTalkUserId(String.valueOf(empInfo.get("emplId"))));
                }
                value = emplsId; // 成员多选
            }
            formData.put(compId, value);
        }

        Map extInfo = UtilMap.map("opUserId, bizObjectJson, isDraft", cUserId, JSON.toJSONString(formData), true);
        Map rsp = cyClient.operateData(ddClient.getAccessToken(), schemaCode, extInfo);
        cyClient.operateData(ddClient.getAccessToken(), schemaCode, UtilMap.map("bizObjectId, opUserId", rsp.get("bizObjectId"), cUserId));

        return McR.success();
    }


    @Autowired
    private DDClient_Storage ddClient_storage;

    @Autowired
    private DDConf ddConf;

    @Autowired
    private DDClient_Contacts ddClient_contacts;

    /**
     * 同步氚云附件到钉盘
     */
    @PostMapping("sync-dp")
    McR syncDP(@RequestBody Map data) {

        Map userInfo = ddClient_contacts.getUserInfoById(ddClient.getAccessToken(), ddConf.getOperator());
        List<Map> spaces = ddClient_storage.getSpaces(ddClient.getAccessToken(), String.valueOf(userInfo.get("unionid")));

        return McR.success();
    }

    @PostMapping("test")
    McR test(HttpServletRequest request) {

        Map data = UtilServlet.getParamMap(request);
        log.info("xxxx, {}", data);

        List<Map> userList = new ArrayList<>();
//
//        List<Map> deptList = cyClient.getDepartmentInfoList(ddClient.getAccessToken(), null);
//        for (Map dept : deptList) {
//            userList.addAll(cyClient.getUserInfoList(ddClient.getAccessToken(), String.valueOf(dept.getDefault("id")), true));
//
//        }
//        log.info("xxxx, {}", userList.size());


        return McR.success(userList);
    }

}
